@@ -10,6 +10,7 @@ from PyQt5.QtWidgets import QFrame, QMessageBox, QMenu, QWidget, QUndoStack, QCh
 
 from urh import settings
 from urh.controller.dialogs.AdvancedModulationOptionsDialog import AdvancedModulationOptionsDialog
+from urh.controller.dialogs.CostaOptionsDialog import CostaOptionsDialog
 from urh.controller.dialogs.FilterDialog import FilterDialog
 from urh.controller.dialogs.SendDialog import SendDialog
 from urh.controller.dialogs.SignalDetailsDialog import SignalDetailsDialog
@@ -105,17 +106,22 @@ class SignalFrame(QFrame):
             self.configure_filter_action.triggered.connect(self.on_configure_filter_action_triggered)
             self.ui.btnFilter.setMenu(self.filter_menu)
 
-            self.auto_detect_menu = QMenu()
-            self.detect_noise_action = self.auto_detect_menu.addAction(self.tr("Additionally detect noise"))
-            self.detect_noise_action.setCheckable(True)
-            self.detect_noise_action.setChecked(False)
-            self.detect_modulation_action = self.auto_detect_menu.addAction(self.tr("Additionally detect modulation"))
-            self.detect_modulation_action.setCheckable(True)
-            self.detect_modulation_action.setChecked(False)
-            self.ui.btnAutoDetect.setMenu(self.auto_detect_menu)
+            if not self.signal.already_demodulated:
+                self.auto_detect_menu = QMenu()
+                self.detect_noise_action = self.auto_detect_menu.addAction(self.tr("Additionally detect noise"))
+                self.detect_noise_action.setCheckable(True)
+                self.detect_noise_action.setChecked(False)
+                self.detect_modulation_action = self.auto_detect_menu.addAction(self.tr("Additionally detect modulation"))
+                self.detect_modulation_action.setCheckable(True)
+                self.detect_modulation_action.setChecked(False)
+                self.ui.btnAutoDetect.setMenu(self.auto_detect_menu)
+
 
             if self.signal.wav_mode:
-                self.ui.lSignalTyp.setText("Signal (*.wav)")
+                if self.signal.already_demodulated:
+                    self.ui.lSignalTyp.setText("Demodulated (1-channel *.wav)")
+                else:
+                    self.ui.lSignalTyp.setText("Signal (*.wav)")
             else:
                 self.ui.lSignalTyp.setText("Complex Signal")
 
@@ -149,6 +155,16 @@ class SignalFrame(QFrame):
 
             self.show_protocol(refresh=False)
 
+            if self.signal.already_demodulated:
+                self.ui.cbModulationType.hide()
+                self.ui.labelModulation.hide()
+                self.ui.labelNoise.hide()
+                self.ui.spinBoxNoiseTreshold.hide()
+                self.ui.btnAutoDetect.hide()
+                self.ui.cbSignalView.setCurrentIndex(1)
+                self.ui.cbSignalView.hide()
+                self.ui.lSignalViewText.hide()
+
         else:
             self.ui.lSignalTyp.setText("Protocol")
             self.set_empty_frame_visibilities()
@@ -261,7 +277,7 @@ class SignalFrame(QFrame):
         self.ui.spinBoxSamplesPerSymbol.setValue(self.signal.samples_per_symbol)
         self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold_relative)
         self.ui.cbModulationType.setCurrentText(self.signal.modulation_type)
-        self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() == "ASK")
+        self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() in ("ASK", "PSK"))
         self.ui.spinBoxCenterSpacing.setValue(self.signal.center_spacing)
         self.ui.spinBoxBitsPerSymbol.setValue(self.signal.bits_per_symbol)
 
@@ -433,19 +449,19 @@ class SignalFrame(QFrame):
 
     def save_signal_as(self):
         try:
-            FileOperator.save_data_dialog(self.signal.name, self.signal.iq_array, self.signal.sample_rate,
-                                          self.signal.wav_mode)
+            FileOperator.ask_signal_file_name_and_save(self.signal.name, self.signal.iq_array, self.signal.sample_rate,
+                                                       self.signal.wav_mode)
         except Exception as e:
             Errors.exception(e)
 
     def export_demodulated(self):
         try:
-            initial_name = self.signal.name + "-demodulated.complex"
+            initial_name = self.signal.name + "-demodulated.wav"
         except Exception as e:
             logger.exception(e)
-            initial_name = "demodulated.complex"
+            initial_name = "demodulated.wav"
 
-        filename = FileOperator.get_save_file_name(initial_name)
+        filename = FileOperator.ask_save_file_name(initial_name, caption="Export demodulated")
         if filename:
             try:
                 self.setCursor(Qt.WaitCursor)
@@ -509,7 +525,10 @@ class SignalFrame(QFrame):
         self.ui.txtEdProto.setText("Demodulating...")
         qApp.processEvents()
 
-        self.proto_analyzer.get_protocol_from_signal()
+        try:
+            self.proto_analyzer.get_protocol_from_signal()
+        except Exception as e:
+            Errors.exception(e)
 
     def show_protocol(self, old_view=-1, refresh=False):
         if not self.proto_analyzer:
@@ -748,8 +767,18 @@ class SignalFrame(QFrame):
     def on_btn_autodetect_clicked(self):
         self.ui.btnAutoDetect.setEnabled(False)
         self.setCursor(Qt.WaitCursor)
-        success = self.signal.auto_detect(detect_modulation=self.detect_modulation_action.isChecked(),
-                                          detect_noise=self.detect_noise_action.isChecked())
+
+        try:
+            detect_modulation = self.detect_modulation_action.isChecked()
+        except AttributeError:
+            detect_modulation = False
+
+        try:
+            detect_noise = self.detect_noise_action.isChecked()
+        except AttributeError:
+            detect_noise = False
+        success = self.signal.auto_detect(detect_modulation=detect_modulation, detect_noise=detect_noise)
+
         self.ui.btnAutoDetect.setEnabled(True)
         self.unsetCursor()
         if not success:
@@ -1068,7 +1097,7 @@ class SignalFrame(QFrame):
                 self.scene_manager.init_scene()
                 self.on_slider_y_scale_value_changed()
 
-        self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() == "ASK")
+        self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() in ("ASK", "PSK"))
 
     @pyqtSlot()
     def on_signal_data_changed_before_save(self):
@@ -1276,9 +1305,25 @@ class SignalFrame(QFrame):
         dialog.message_length_divisor_edited.connect(self.on_message_length_divisor_edited)
         return dialog
 
+    def get_costas_dialog(self):
+        dialog = CostaOptionsDialog(self.signal.costas_loop_bandwidth, parent=self)
+        dialog.accepted.connect(self.on_costas_dialog_accepted)
+        return dialog
+
+    @pyqtSlot()
+    def on_costas_dialog_accepted(self):
+        sender = self.sender()
+        assert isinstance(sender, CostaOptionsDialog)
+        self.signal.costas_loop_bandwidth = sender.costas_loop_bandwidth
+
     @pyqtSlot()
     def on_btn_advanced_modulation_settings_clicked(self):
-        dialog = self.get_advanced_modulation_settings_dialog()
+        if self.ui.cbModulationType.currentText() == "ASK":
+            dialog = self.get_advanced_modulation_settings_dialog()
+        elif self.ui.cbModulationType.currentText() == "PSK":
+            dialog = self.get_costas_dialog()
+        else:
+            raise ValueError("No additional settings available")
         dialog.exec_()
 
     @pyqtSlot()
@@ -1289,7 +1334,7 @@ class SignalFrame(QFrame):
             logger.exception(e)
             initial_name = "spectrogram.ft"
 
-        filename = FileOperator.get_save_file_name(initial_name, caption="Export spectrogram")
+        filename = FileOperator.ask_save_file_name(initial_name, caption="Export spectrogram")
         if not filename:
             return
         QApplication.setOverrideCursor(Qt.WaitCursor)
